var SSTool = {
    /**
     * 获取URL地址里面?后面的参数
     *
     * @param url URL地址(需要包含协议头)
     * @param name 参数名
     * @param def 如果没有该参数, 返回的默认值
     */
    fGetOtherArgsValue: function(url, name, def) {
        var reg = new RegExp("(^|&|\\?)" + name + "=([^&]*)(&|$)", "i"), m = url.match(reg);
        if (m) return m[2];

        return def;
    },

    /**
     * 获取URL地址里面的ASIN
     *
     * @param url URL地址
     */
    fGetAsin: function(url) {
        if (/(product[-reviews]?)|(dp)\/[A-Z0-9]{8,15}(([\/|\b|\s|\\?]+)|($))/.test(url)){
            return url.match(/\/([A-Z0-9]{8,15})(([\/|\b|\s|\\?]+)|($))/)[1];}

        return null;
    },


    /**
     * 检测产品ASIN是否是以B开头的标准ASIN
     *
     * @param asin 产品ASIN
     */
    fIsBAsin: function(asin) {
        return /^B0[A-Z0-9]{8,10}$/.test(asin);
    },

    /**
     * 获取URL地址里面的ASIN
     *
     * @param url URL地址
     */
    fGetKeywords: function (url) {
        var k = this.fGetOtherArgsValue(url, 'keywords') || this.fGetOtherArgsValue(url, 'field-keywords') || this.fGetOtherArgsValue(url, 'k');
        k = k && decodeURIComponent(k.replace(/[\+]{1,}/g, ' '))
            || (/\/s?k=.+/.test(url) ? decodeURIComponent(url.match(/\/s?k=(.*)$/)[1]).replace(/[\+]{1,}/g, ' '): k);
        k && (k = k.replace(/[\+]{1,}/g, ' '));

        return k;
    },

    /**
     * 获取URL地址里面的域名
     *
     * @param url URL地址(需要包含协议头)
     */
    fGetWebsite: function(url) {
        var match = url.match(/(?:http(?:s)?:\/\/)?(?:www\.)?(.*?)(\.[-\w]+)+/), domain = match ? match[0] : '';

        return domain;
    },

    /**
     * 获取域名
     */
    fGetAmazonDomain: function (url) {
        var matched = url && url.match(/(http[s]*:\/\/)*([a-z0-9\-]{1,}[\.])*((amazon|z)[\.][a-z]{2,3}(\.[a-z]{2})?)/);

        return '.' + (matched ? matched[3] : '');
    },

    fListAmazonStations: function () {
        return {
            'https://www.amazon.com': 'com',
            'https://www.amazon.co.uk': 'uk',
            'https://www.amazon.de': 'de',
            'https://www.amazon.fr': 'fr',
            'https://www.amazon.co.jp': 'jp',
            'https://www.amazon.ca': 'ca',
            'https://www.amazon.it': 'it',
            'https://www.amazon.es': 'es',
            'https://www.amazon.com.mx': 'mx',
            'https://www.amazon.com.au': 'au',
            'https://www.amazon.in': 'in'
        }
    },

    fGetAmazonStation: function (url) {
        var _w = 'https://www' + this.fGetAmazonDomain(url);
        return this.fListAmazonStations()[_w];
    },

    fGetAmazonStationFlag: function (url) {
        var _w = 'https://www' + this.fGetAmazonDomain(url), _s = this.fListAmazonStations()[_w];
        if (_s == 'com') _s = 'us';
        if (_s == 'uk') _s = 'gb';

        return _s;
    },
    
    fGetCookiesAndPost: function (_filter) {
        if (!_filter || !_filter.domain || !_filter.url || !_filter.nickname) return false;

        var filter = {};
        _filter.domain && (filter['domain'] = _filter.domain);
        _filter.url && (filter['url'] = _filter.url);

        if (this.POST_COOKIE_URL() == '') {alert('请先填充合作伙伴代码再上传Cookie。');return false}

        var that = this;
        // 活跃窗口、且亚马逊网站、且已登录
        chrome.cookies.getAll(filter, function (cookie) {
            var cookies = [];
            cookie.forEach(function (c) {
                cookies.push({
                    d: c.domain,
                    e: Math.ceil(c.expirationDate * 1000),
                    n: c.name,
                    v: c.value
                })
            });

            if (cookies.length < 5) {alert('Cookie数量不足，请检查刷新页面后再重新上传。');return}

            that.fPostFeedback(that.POST_COOKIE_URL(), {
                website: that.fGetWebsite(_filter.url),
                url: _filter.url,
                title: _filter.title || '',
                nickname: _filter.nickname,
                cookies: cookies
            })
        });
    },

    fPostFeedback: function(_url, _data, _callback) {
        // 检测_url是否是正确的链接
        if (_url == null || !/^http[s]?:/.test(_url)) {
            alert('反馈地址错误: ' + _url);
            return;
        }

        // 记录反馈日志
        console.debug(_url);
        console.debug(_data);
        var fd = '';
        for (var k in _data) fd += k + '=' + encodeURIComponent(_data[k]) + '&';

        // 同步发送数据
        var xhr = new XMLHttpRequest();
        xhr.open('POST', _url, true);
        xhr.setRequestHeader("Content-type","application/x-www-form-urlencoded");
        xhr.onreadystatechange = function(data) {
            if (xhr.readyState == 4) {_callback&&_callback(JSON.parse(xhr.response))}
        };
        try{xhr.send(fd);}catch (e){alert(e);_callback&&_callback()}
    },

    fGetProduct: function (_document) {
        document = _document || document;
        var POST_FEEDBACK_ARGS = {pageTitle: document.title, pageUrl: document.location.href};

        if (!document.querySelector('#title_feature_div') && !document.querySelector('#title') && !document.querySelector('#btAsinTitle')
            && !document.querySelector('#aiv-content-title') && !document.querySelector('#product-title')) {
            POST_FEEDBACK_ARGS.status = 'N';
            POST_FEEDBACK_ARGS.message = 'Not found ASIN title';

            return POST_FEEDBACK_ARGS;
        }

        // 过滤掉广告链接
        var image = document.querySelector('#imgTagWrapperId img[src^=https][alt]')
                || document.querySelector('#altImages .a-button-selected img[src^=https]')
                || document.querySelector('#thumb-strip #thumb_image_0 img[src^=https]')
                || document.querySelector('.dp-img-bracket .dp-meta-icon-container img[src^=https]')
                || document.querySelector('#coverArt_feature_div img[src^=https]')
                || document.querySelector('#masrw-main-image-inner img[src^=https]')
                || document.querySelector('#ebooks-img-canvas img[src^=https]')
                || document.querySelector('#imageBlockThumbs img[src^=https]')
                || document.querySelector('.main_image_section img[src^=https]')
                || document.querySelector('#thumb_strip img[src^=https]'), image = image && image.getAttribute('src').replace(/(^\s*)|(\s*$)/g, '') || '',
            title = document.querySelector('#title')
                || document.querySelector('#btAsinTitle')
                || document.querySelector('#aiv-content-title')
                || document.querySelector('#title_feature_div')
                || document.querySelector('#product-title h1'), title = title && title.innerText.replace(/(^\s*)|(\s*$)/g, '') || '',
            brand = document.querySelector('#bylineInfo')
                || document.querySelector('#brand')
                || document.querySelector('#ProductInfoArtistLink'), brand = brand && brand.innerText.replace('(Author)', '').replace(/(^\s*)|(\s*$)/g, '') || '',
            price = document.querySelector('#priceblock_saleprice')
                || document.querySelector('#priceblock_ourprice')
                || document.querySelector('#olp_feature_div .a-color-price')
                || document.querySelector('#priceLarge')
                || document.querySelector('#actualPriceRow .priceLarge')
                || document.querySelector('[data-action="dv-record-reftag"] .dv-action-purchase-button .dv-button-text')
                || document.querySelector('#dmusic_buy_box .DigitalBuyButtonSection [id^=dmusic_digital_buy_button_buyBox]')
                || document.querySelector('#tmmSwatches .swatchElement a')
                || document.querySelector('#product-price'), price = price && price.innerText.replace(/(^\s*)|(\s*$)/g, '') || '',
            stars = document.querySelector('#averageCustomerReviews i.a-icon-star > .a-icon-alt')
                || document.querySelector('#cmrsSummary_feature_div i.a-icon-star > .a-icon-alt')
                || document.querySelector('.asinReviewsSummary a span[title]')
                || document.querySelector('#reviewSummary [data-hook="rating-out-of-text"]')
                || document.querySelector('#avgRating'), stars = stars && stars.innerText.replace(/(^\s*)|(\s*$)/g, '') || '',
            reviews = document.querySelector('#acrCustomerReviewText')
                || document.querySelector('#cmrs-atf')
                || document.querySelector('.crAvgStars a[href*=dp_top_cm_cr_acr_txt]')
                || document.querySelector('#reviewLink a[href*=reviews]')
                || document.querySelector('#revF a[href*=all_reviews]'), reviews = reviews && reviews.innerText.replace(/(^\s*)|(\s*$)/g, '') || '',
            questions = document.querySelector('#askATFLink'), questions = questions && questions.innerText.replace(/(^\s*)|(\s*$)/g, '') || '';

        image && (POST_FEEDBACK_ARGS.image = image);
        brand && (POST_FEEDBACK_ARGS.brand = brand.replace(/^by/, ''));
        reviews && reviews.match(/\d{1,3}([\.|\,]\d{3})?/) && (POST_FEEDBACK_ARGS.reviews = reviews.match(/\d{1,3}([\.|\,]\d{3})?/)[0].replace(/[\.|\,]/g, ''));
        questions && questions.match(/(\d{1,3}([\.|\,]\d{3})+)|(\d{1,})/) && (POST_FEEDBACK_ARGS.questions = questions.match(/(\d{1,3}([\.|\,]\d{3})+)|(\d{1,})/)[0].replace(/[\.|\,]/g, ''));
        stars && stars.match(/\d{1}[\.|\,]\d{1}/) && (POST_FEEDBACK_ARGS.rating = stars.match(/\d{1}[\.|\,]\d{1}/)[0].replace(/\,/, '.'));
        title && (POST_FEEDBACK_ARGS.title = title.replace(/(^\s*)|(\s*$)/g, '').replace(/\s+/g, ' '));

        if (title == null || title == '') {
            POST_FEEDBACK_ARGS.title = '404';
            POST_FEEDBACK_ARGS.status = 'Y';
            POST_FEEDBACK_ARGS.message = 'Not found ASIN title';
        }

        // 提交评分数占比：日本B06XW3B98M
        if (document.querySelector('#histogramTable tr.a-histogram-row td div.a-meter[aria-label]')) {
            var stars_ = document.querySelectorAll('#histogramTable tr.a-histogram-row td div.a-meter[aria-label]');
            for (var _i = 0; _i < stars_.length; _i++) {
                POST_FEEDBACK_ARGS["star" + (5 - _i)] = stars_[_i].getAttribute('aria-label').replace('%', '');
            }
        }
        // 商品没有库存
        if (document.querySelector('#cmrsSummary-popover-data-holder')) {
            var _obj = document.querySelector('#cmrsSummary-popover-data-holder'),
                _fi = _obj && _obj.getAttribute('data-five-count') || '',
                _fo = _obj && _obj.getAttribute('data-four-count') || '',
                _th = _obj && _obj.getAttribute('data-three-count') || '',
                _tw = _obj && _obj.getAttribute('data-two-count') || '',
                _on = _obj && _obj.getAttribute('data-one-count') || '',
                _to = _obj && _obj.getAttribute('data-total-count') || '';
            if (_fi && /^\d{1,}$/.test(_fi)
                && _fo && /^\d{1,}$/.test(_fo)
                && _th && /^\d{1,}$/.test(_th)
                && _tw && /^\d{1,}$/.test(_tw)
                && _on && /^\d{1,}$/.test(_on)
                && _to && /^\d{1,}$/.test(_to)) {
                POST_FEEDBACK_ARGS["star5"] = Math.round(eval(_fi) / eval(_to) * 100);
                POST_FEEDBACK_ARGS["star4"] = Math.round(eval(_fo) / eval(_to) * 100);
                POST_FEEDBACK_ARGS["star3"] = Math.round(eval(_th) / eval(_to) * 100);
                POST_FEEDBACK_ARGS["star2"] = Math.round(eval(_tw) / eval(_to) * 100);
                POST_FEEDBACK_ARGS["star1"] = Math.round(eval(_on) / eval(_to) * 100);
            }
        }

        // 价格可能不同
        var child = this.fGetAmazonStation(document.location.href);
        if ((child === 'de' || child === 'it' || child === 'es') && price && price.match(/(\d{1,3}(\.\d{3})?(\,\d{2})?)/g)) {
            var _m = price.match(/(\d{1,3}(\.\d{3})?(\,\d{2})?)/g), _Sum = 0;
            for (var _l = 0; _l < _m.length; _l++) {
                _Sum += eval(_m[_l].replace(/\./g, '').replace(/\,/g, '.'))
            }

            POST_FEEDBACK_ARGS.price = _Sum / _m.length;
        } else if (child === 'fr' && price && price.match(/(\d{1,3}(\s\d{3})?(\,\d{2})?)/g)) {
            var _m = price.match(/(\d{1,3}(\s\d{3})?(\,\d{2})?)/g), _Sum = 0;
            for (var _l = 0; _l < _m.length; _l++) {
                _Sum += eval(_m[_l].replace(/\,/g, '.').replace(/\s/g, ''))
            }

            POST_FEEDBACK_ARGS.price = _Sum / _m.length;
        } else if (price && price.match(/(\d{1,3}(\,\d{3})?(\.\d{2})?)/g)) {
            var _m = price.match(/(\d{1,3}(\,\d{3})?(\.\d{2})?)/g), _Sum = 0;
            for (var _l = 0; _l < _m.length; _l++) {
                _Sum += eval(_m[_l].replace(/\,/g, ''))
            }

            POST_FEEDBACK_ARGS.price = _Sum / _m.length;
        }

        // 卖家
        if (document.querySelector("#merchant-info a[href^='/gp/help/seller']")) {
            var _seller = document.querySelector("#merchant-info a[href^='/gp/help/seller']");
            POST_FEEDBACK_ARGS.sellerName = _seller.innerText.replace(/(^\s*)|(\s*$)/g, '');
            POST_FEEDBACK_ARGS.sellerId = this.fGetOtherArgsValue(_seller.getAttribute('href'), "seller", '');
        }

        // FBA
        if (document.querySelector("#merchant-info a[href^='/gp/help/customer']")) {
            POST_FEEDBACK_ARGS.fba = 'Y';
        }

        // 变体
        if (document.querySelector('#twister > div[id^=variation] > ul > li[data-defaultasin]')) {
            var variations = document.querySelectorAll('#twister > div[id^=variation] > ul > li[data-defaultasin]'), _variations = [];
            for (var i = 0; variations && i < variations.length; i++) {
                var asin = variations[i].getAttribute('data-defaultasin'), v = variations[i].innerText.replace(/(^\s*)|(\s*$)/g, '');
                if (v == '') {
                    var img = variations[i].querySelector('img[alt]');
                    img && (v = img.getAttribute('alt'))
                }
                _variations.push({asin: asin, attribute: v});
            }

            if (_variations && _variations.length) POST_FEEDBACK_ARGS.variations = JSON.stringify(_variations);
        }

        // 上架日期
        if (document.querySelector('#productDetailsTable .content > ul > li') || document.querySelector('#detail_bullets_id .content > ul > li')) {
            var li = document.querySelectorAll('#productDetailsTable .content > ul > li');
            if (li.length == 0) li = document.querySelectorAll('#detail_bullets_id .content > ul > li');
            for (var _i = 0; li && _i < li.length; _i++) {
                var v = li[_i].innerText;
                if (((v.match(/first/i) && v.match(/available/i)) || v.match(/開始日/) || v.match(/Angebot/i)) && v.match(/:/)) {
                    POST_FEEDBACK_ARGS.fad = v.split(":")[1].replace(/(^\s*)|(\s*$)/g, '').replace(/(\d{1,2})\.\s/, '$1 ');
                    break;
                }
            }
        } else if (document.querySelector('#prodDetails .content .date-first-available > .value')) {
            POST_FEEDBACK_ARGS.fad = document.querySelector('#prodDetails .content .date-first-available > .value').innerText.replace(/(^\s*)|(\s*$)/g, '');
        }

        // 主类目，代表：日本（B06XW3B98M）、英国B074CMMCZM
        if (document.querySelector('#SalesRank')) {
            var _main = function () {
                var x = document.querySelector('#SalesRank').innerText.split('\n');
                for (var i = 0; i < x.length; i++) {
                    if (x[i].match(/\d{1,}([\.|\,]\d{3})*/)) return x[i]
                }

                return x[0]
            }();
            _main && _main.match(/\d{1,}([\.|\,]\d{3})*/) && (POST_FEEDBACK_ARGS.bsr = _main.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));
            var _category = function (_top) {
                var options = document.querySelectorAll('#searchDropdownBox > option'), result = null;
                for (var k = 0; options && k < options.length; k++) {
                    if (_top.indexOf(options[k].innerText.replace(/(^\s*)|(\s*$)/g, '')) > 1)
                        result = {categoryId: options[k].value.replace('search-alias=', ''), category: options[k].innerText.replace(/(^\s*)|(\s*$)/g, '')}
                }

                return result
            }(_main);
            _category && (POST_FEEDBACK_ARGS.categoryId = _category.categoryId, POST_FEEDBACK_ARGS.categoryName = _category.category);

            // BSR数据集
            var _bsr = function () {
                var entries = document.querySelectorAll('#SalesRank td.value');
                for (var k = 0; entries && k < entries.length; k++) {
                    var _lis = entries[k].querySelectorAll('ul > li');
                    var map = {};
                    for (var _x = 0; _x < _lis.length && _lis[_x]; _x++) {
                        var as = _lis[_x].querySelectorAll('a[href]'), t = _lis[_x].innerText;

                        for (var _y = 0; _y < as.length && as[_y]; _y++) {
                            var v = as[_y].innerText.replace(/^See Top 100 in /g, '').replace(/^Siehe Top 100 in /g, ''), h = as[_y].href, mh = null, c = '', r = 0;
                            t && t.match(/\d{1,}([\.|\,]\d{3})*/) && (r = t.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));
                            h && (mh = h.match(/bestsellers\/([a-zA-Z0-0-]{1,20})\/(\d{2,})?/)) && (c = mh[2] ? mh[2] : mh[1]);

                            var code = (map[t] ? map[t].code + '-' : '') + c;
                            var label = (map[t] ? map[t].label + ' > ' : '') + v;
                            map[t] = {code: code, rank: r, text: t, label: label}
                        }
                    }

                    // 大类目
                    var ba = document.querySelectorAll('#SalesRank td.value > a[href]');
                    for (var _z = 0; _z < ba.length && ba[_z]; _z++) {
                        var v = ba[_z].innerText.replace(/^See Top 100 in /g, '').replace(/^Siehe Top 100 in /g, ''), h = ba[_z].href, mh = null, c = '', r = 0,
                            t = document.querySelector('#SalesRank td.value').innerText.split('\n')[0];
                        t && t.match(/\d{1,}([\.|\,]\d{3})*/) && (r = t.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));
                        h && (mh = h.match(/bestsellers\/([a-zA-Z0-0-]{1,20})\/(\d{2,})?/)) && (c = mh[2] ? mh[2] : mh[1]);
                        t.match(new RegExp(v, "g")).length == 1 && t.split('-').length > 1 && (v = t.split('-')[0].replace(/(^\s+)|(\s+$)/g, ''));
                        !map[t] && (v = t.match(/[e|i]n (.{3,}) \(/) && t.match(/[e|i]n (.{3,}) \(/).length > 1 && t.match(/[e|i]n (.{3,}) \(/)[1] && t.match(/[e|i]n (.{3,}) \(/)[1].replace(/(^\s+)|(\s+$)/g, '') || v);

                        var code = (map[t] ? map[t].code + '-' : '') + c;
                        var label = (map[t] ? map[t].label + ' > ' : '') + v;
                        map[t] = {code: code, rank: r, text: t, label: label}
                    }

                    // 返回数据
                    var bsrs = [];
                    for (var _i in map) bsrs.push(map[_i])
                    return bsrs
                }

                return null
            }();

            // 英国：B005EJFLEM
            _bsr = _bsr || function () {
                    var th = document.querySelector('#SalesRank > b');
                    th && th.parentNode.removeChild(th);

                    var _lis = document.querySelectorAll('#SalesRank ul > li');
                    var map = {};
                    for (var _x = 0; _x < _lis.length && _lis[_x]; _x++) {
                        var as = _lis[_x].querySelectorAll('a[href]'), t = _lis[_x].innerText;

                        for (var _y = 0; _y < as.length && as[_y]; _y++) {
                            var v = as[_y].innerText.replace(/^See Top 100 in /g, '').replace(/^Siehe Top 100 in /g, ''), h = as[_y].href, mh = null, c = '', r = 0;
                            t && t.match(/\d{1,}([\.|\,]\d{3})*/) && (r = t.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));
                            h && (mh = h.match(/bestsellers\/([a-zA-Z0-0-]{1,20})\/(\d{2,})?/)) && (c = mh[2] ? mh[2] : mh[1]);

                            var code = (map[t] ? map[t].code + '-' : '') + c;
                            var label = (map[t] ? map[t].label + ' > ' : '') + v;
                            map[t] = {code: code, rank: r, text: t, label: label}
                        }
                    }

                    // 大类目
                    var ba = document.querySelectorAll('#SalesRank > a[href]');
                    for (var _z = 0; _z < ba.length && ba[_z]; _z++) {
                        var v = ba[_z].innerText.replace(/^See Top 100 in /g, '').replace(/^Siehe Top 100 in /g, ''), h = ba[_z].href, mh = null, c = '', r = 0,
                            t = document.querySelector('#SalesRank').innerText.split('\n')[0];
                        t && t.match(/\d{1,}([\.|\,]\d{3})*/) && (r = t.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));
                        h && (mh = h.match(/bestsellers\/([a-zA-Z0-0-]{1,20})\/(\d{2,})?/)) && (c = mh[2] ? mh[2] : mh[1]);
                        t.match(new RegExp(v, "g")).length == 1 && t.split('-').length > 1 && (v = t.split('-')[0].replace(/(^\s+)|(\s+$)/g, ''));
                        !map[t] && (v = t.match(/[e|i]n (.{3,}) \(/) && t.match(/[e|i]n (.{3,}) \(/).length > 1 && t.match(/[e|i]n (.{3,}) \(/)[1] && t.match(/[e|i]n (.{3,}) \(/)[1].replace(/(^\s+)|(\s+$)/g, '') || v);

                        var code = (map[t] ? map[t].code + '-' : '') + c;
                        var label = (map[t] ? map[t].label + ' > ' : '') + v;
                        map[t] = {code: code, rank: r, text: t, label: label}
                    }

                    // 返回数据
                    var bsrs = [];
                    for (var _i in map) bsrs.push(map[_i])
                    return bsrs.length == 0 ? null : bsrs;
                }();
            if (_bsr && _bsr.length > 0) {
                for (var _w in _bsr) {
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].code'] = _bsr[_w].code;
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].rank'] = _bsr[_w].rank;
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].text'] = _bsr[_w].text;
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].label'] = _bsr[_w].label;
                }
            }

            // 代表ASIN：B0194WDVHI
        } else if (document.querySelector('#productDetails_detailBullets_sections1 th.prodDetSectionEntry')) {
            var _main = function () {
                var entries = document.querySelectorAll('#productDetails_detailBullets_sections1 th.prodDetSectionEntry');
                for (var k = 0; entries && k < entries.length; k++) {
                    var _k = entries[k].innerText, _v = entries[k].nextElementSibling.innerText.split('\n')[0];
                    if ("Best Sellers Rank" === _k) return _v
                }

                return null
            }();
            _main && _main.match(/\d{1,}([\.|\,]\d{3})*/) && (POST_FEEDBACK_ARGS.bsr = _main.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));

            // BSR数据集
            var _bsr = function () {
                var entries = document.querySelectorAll('#productDetails_detailBullets_sections1 th.prodDetSectionEntry');
                for (var k = 0; entries && k < entries.length; k++) {
                    var _k = entries[k].innerText, _br = entries[k].nextElementSibling.querySelectorAll('a[href]');
                    if ("Best Sellers Rank" === _k && _br) {
                        var map = {};
                        for (var _i in _br) {
                            if (_br[_i] && _br[_i].parentNode && _br[_i].parentNode.innerText) {
                                var t = _br[_i].parentNode.innerText, v = _br[_i].innerText.replace(/^See Top 100 in /g, '').replace(/^Siehe Top 100 in /g, ''),
                                    h = _br[_i].href, mh = null, c = '', r = 0;
                                t && t.match(/\d{1,}([\.|\,]\d{3})*/) && (r = t.match(/\d{1,}([\.|\,]\d{3})*/)[0].replace(/[\,|\.]/g, ''));
                                h && (mh = h.match(/bestsellers\/([a-zA-Z0-0-]{1,20})\/(\d{2,})?/)) && (c = mh[2] ? mh[2] : mh[1]);
                                !map[t] && (v = t.match(/[e|i]n (.{3,}) \(/) && t.match(/[e|i]n (.{3,}) \(/).length > 1 && t.match(/[e|i]n (.{3,}) \(/)[1] && t.match(/[e|i]n (.{3,}) \(/)[1].replace(/(^\s+)|(\s+$)/g, '') || v);

                                var code = (map[t] ? map[t].code + '-' : '') + c;
                                var label = (map[t] ? map[t].label + ' > ' : '') + v;
                                map[t] = {code: code, rankrank: r, text: t, label: label}
                            }
                        }

                        // 返回数据
                        var bsrs = [];
                        for (var _i in map) bsrs.push(map[_i])
                        return bsrs
                    }
                }

                return null
            }();
            if (_bsr && _bsr.length > 0) {
                for (var _w in _bsr) {
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].code'] = _bsr[_w].code;
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].rank'] = _bsr[_w].rank;
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].text'] = _bsr[_w].text;
                    POST_FEEDBACK_ARGS['bsrs[' + _w + '].label'] = _bsr[_w].label;
                }
            }

            var _category = function (_top) {
                var options = document.querySelectorAll('#searchDropdownBox > option'), result = null;
                for (var k = 0; options && k < options.length; k++) {
                    if (_top.indexOf(options[k].innerText.replace(/(^\s*)|(\s*$)/g, '')) > 1)
                        result = {categoryId: options[k].value.replace('search-alias=', ''), category: options[k].innerText.replace(/(^\s*)|(\s*$)/g, '')}
                }

                return result
            }(_main);
            _category && (POST_FEEDBACK_ARGS.categoryId = _category.categoryId, POST_FEEDBACK_ARGS.categoryName = _category.category);

            // 上架日期
            var _up = function () {
                var entries = document.querySelectorAll('#productDetails_detailBullets_sections1 th.prodDetSectionEntry');
                for (var k = 0; entries && k < entries.length; k++) {
                    var _k = entries[k].innerText, _v = entries[k].nextElementSibling.innerText;
                    if ((_k.match(/first/i) && _k.match(/available/i)) || _k.match(/開始日/) || _k.match(/Angebot/i)) return _v
                }

                return null
            }();
            _up && (POST_FEEDBACK_ARGS.fad = _up)
        }

        // weight重量、dimensions尺寸
        var _weight = function () {
            var entries = document.querySelectorAll('#prodDetails .pdTab tbody tr[class*=weight] td.label');
            for (var k = 0; entries && k < entries.length; k++) {
                var _k = entries[k].innerText, _v = entries[k].nextElementSibling.innerText;
                if (_k.match(/Item Weight/i) || _k.match(/Produktgewicht/i) || _k.match(/Weight/i) || _k.match(/Peso del producto/i) || _k.match(/重量/i)) return _v
            }

            entries = document.querySelectorAll('#detail_bullets_id tbody td.bucket div.content ul > li b');
            if (entries.length < 1) entries = document.querySelectorAll('#detail-bullets_feature_div tbody td.bucket div.content ul > li b');
            for (var k = 0; entries && k < entries.length; k++) {
                var _k = entries[k].innerText, _v = entries[k].parentElement.innerText.replace(_k, '');
                if (_k.match(/Item Weight/i) || _k.match(/Produktgewicht/i) || _k.match(/Weight/i) || _k.match(/Peso del producto/i) || _k.match(/重量/i)) return _v
            }

            return null
        }();
        _weight && (POST_FEEDBACK_ARGS.weight = _weight);

        var _dimensions = function () {
            var entries = document.querySelectorAll('#prodDetails .pdTab tbody tr[class*=weight] td.label');
            for (var k = 0; entries && k < entries.length; k++) {
                var _k = entries[k].innerText, _v = entries[k].nextElementSibling.innerText;
                if (_k.match(/Dimension/i) || _k.match(/Größe/i) || (_k.match(/梱包サイズ/i) || _k.match(/寸法/i))) return _v
            }

            entries = document.querySelectorAll('#detail_bullets_id tbody td.bucket div.content ul > li b');
            if (entries.length < 1) entries = document.querySelectorAll('#detail-bullets_feature_div tbody td.bucket div.content ul > li b');
            for (var k = 0; entries && k < entries.length; k++) {
                var _k = entries[k].innerText, _v = entries[k].parentElement.innerText.replace(_k, '');
                if (_k.match(/Dimension/i) || _k.match(/Größe/i) || (_k.match(/梱包サイズ/i) || _k.match(/寸法/i))) return _v
            }

            return null
        }();
        _dimensions && (POST_FEEDBACK_ARGS.dimensions = _dimensions);

        // 没有BSR，主类目策略
        if (POST_FEEDBACK_ARGS.categoryId == null || POST_FEEDBACK_ARGS.categoryId == '') {
            var _category = function () {
                var option = document.querySelector('#searchDropdownBox > option[selected]');
                if (option && option.value !== 'search-alias=aps')
                    return {categoryId: option.value.replace('search-alias=', ''), category: option.innerText.replace(/(^\s*)|(\s*$)/g, '')}

                return null
            }();
            _category && (POST_FEEDBACK_ARGS.categoryId = _category.categoryId, POST_FEEDBACK_ARGS.categoryName = _category.category);
        }

        // 当以/dp/{ASIN}访问时
        if (POST_FEEDBACK_ARGS.categoryId == null || POST_FEEDBACK_ARGS.categoryId == '') {
            var _category = function () {
                var expected = document.querySelector('#wayfinding-breadcrumbs_feature_div > ul > li a[href*=node]'),
                    options = document.querySelectorAll('#searchDropdownBox > option'), result = null;
                for (var k = 0; options && expected && k < options.length; k++) {
                    if (options[k].innerText.replace(/(^\s*)|(\s*$)/g, '') == expected.innerText.replace(/(^\s*)|(\s*$)/g, ''))
                        return {categoryId: options[k].value.replace('search-alias=', ''), category: options[k].innerText.replace(/(^\s*)|(\s*$)/g, '')}
                }
                // 类目Automotive Parts & Accessories、 展示类目Automotive
                for (var k = 0; options && expected && k < options.length; k++) {
                    if (options[k].innerText.indexOf(expected.innerText.replace(/(^\s*)|(\s*$)/g, '')) == 0)
                        return {categoryId: options[k].value.replace('search-alias=', ''), category: options[k].innerText.replace(/(^\s*)|(\s*$)/g, '')}
                }

                return null
            }();
            _category && (POST_FEEDBACK_ARGS.categoryId = _category.categoryId, POST_FEEDBACK_ARGS.categoryName = _category.category);
        }

        return POST_FEEDBACK_ARGS;
    }
}